import common

def flow(method):
    """
    Decorator which denotes that a method is a flow method. You must explicitly
    add the exposed decorator in addition to this one for security reasons.

    A flow is just a parameterless method which uses generators to yield    
    """
    flowid = common._get_flowid()
    def _m(self, _resume=0, *args, **kwargs):
        # this is the actual cherrypy handler function
        # _resume is the magic variable which lets the server know
        # at which point in the flow the client is resuming.
        _resume = int(_resume)
        gen = common._get_gen(flowid, _resume, method, self)
        result = gen.next() # execute the next resume point
        common._save_resumepoint(flowid, gen)
        return result # and return the response
    return _m
