from cherrypy import cpg
from cherrypy.lib.filter import generatorfilter

import cherryflow

class CheckOut:
    def sellItem(self):
        #This is a translation of the devworks article code, note that
        #there is no error checking (which is greatly simplified by
        #continuations, see other examples)

        #Send off and parse first page
        yield Pages.GET_RATE_AMT % {'resume': cpg.request.resume_id}
        rate = int(cpg.request.paramMap['rate'])
        qty = int(cpg.request.paramMap['qty'])
        amount = rate*qty

        #Send off second page and do some business logic on results
        yield Pages.GET_ZONE % {'resume': cpg.request.resume_id,
                                'qty': qty, 'rate': rate}
        zone = cpg.request.paramMap['zone']
        delOpt = cpg.request.paramMap['delOpt']
        if 'A' == zone and qty >= 100:
            discount = 0.1
        elif 'B' == zone and qty >= 200:
            discount = 0.2
        else:
            discount = 0 #the article had 0.2 here, but that makes no sense
        discrate = 100 * discount
        savings = discount * amount

        #Optionally fire off a third page
        delCost = 0
        if 'S' == delOpt:
            yield Pages.GET_SHIP_OPT % {'resume': cpg.request.resume_id}
            delCost = int(cpg.request.paramMap['delCost'])

        #do some more business logic and show confirmation page
        total = amount + delCost - savings
        yield Pages.DISPLAY_RESULT % {'discrate':discrate, 'total':total,
                                      'savings':savings, 'delCost':delCost,
                                      'amount':amount, 'discount':discount,
                                      'zone':zone}

    #Expose the index and enable flow control.
    index = cpg.expose(cherryflow.flow(sellItem)) # py2.3

class Pages:
    #These are here because this is a translation of the devworks tutorial.
    #There are much better ways to do all this in Subway.
    #We aren't even using a templating system here.
    GET_RATE_AMT = """
<html>
    <head><title>Get Rate and Quantity of item to be purchased</title></head>
    <body>
        <form method="post" action="/?_resume=%(resume)d">
            <p>Enter Rate: <input type="text" name="rate"/></p>
            <p>Enter Quantity: <input type="text" name="qty"/></p>
            <input type="submit" name="submit" value="Next"/>
        </form>
    </body>
</html>""".lstrip()
    GET_ZONE = """
<html>
    <head><title>Get Buyer Category and Delivery Option</title></head>
    <body>
      <form method="post" action="/?_resume=%(resume)d">
        <p>You are buying %(qty)s items, at %(rate)s apiece</p>
        <p>Please specify your category:<br/> 
            <input type="radio" name="zone" value="A">A</input><br/>
            <input type="radio" name="zone" value="B">B</input><br/>
            <input type="radio" name="zone" value="C">C</input><br/>
        </p>
        <p>Will you be picking up the item yourself from our warehouse, or would you like it shipped?<br/>
            <input type="radio" name="delOpt" value="P">Pickup</input><br/>
            <input type="radio" name="delOpt" value="S">Shipping</input><br/>
        </p>
        <input type="submit" name="submit" value="Next"/>
      </form>
    </body>
</html>""".lstrip()
    GET_SHIP_OPT = """
<html>
    <head><title>Get Zone for buyer</title></head>
    <body>
      <form method="post" action="/?_resume=%(resume)d">
        <p>Please specify the shipping option:<br/> 
            <input type="radio" name="delCost" value="100">Standard Delivery</input><br/>
            <input type="radio" name="delCost" value="200">Express Delivery</input><br/>
        </p>
        <input type="submit" name="submit" value="Next"/>
      </form>
    </body>
</html>""".lstrip()
    DISPLAY_RESULT = """
<html>
    <head><title>Total Amount to be paid</title></head>
    <body>
      <form action="./" method="post">
        <p>
        Your shipping costs are %(delCost)s.<br/>
        Your total amount is %(amount)s.<br/>
        You got a discount of %(discrate)s%%.<br/>
        You have to pay %(total)s, a savings of %(savings)s overall.<br/>
        </p>
        <input type="submit" name="submit" value="Start over"/>
      </form>
    </body>
</html>""".lstrip()

if __name__ == "__main__":
    cpg.root = CheckOut()
    cpg.server.start(configDict={'socketPort':8080,'sessionStorageType':'ram'})
